@@ -13,19 +13,8 @@ for token in xmlparser::Tokenizer::from("<tagname name='value'/>") {
 
 ## Why a new library
 
-The main idea of this library is to provide a fast, low-level and complete XML parser.
-
-Unlike other XML parsers, this one can return tokens not with `&str`/`&[u8]` data, but
-with `StrSpan` objects, which contain a position of the data in the original document.
-Which can be very useful if you want to post-process tokens even more and want to return
-errors with a meaningful position.
-
-So, this is basically an XML parser framework that can be used to write parsers for XML-based formats,
-like SVG and to construct a DOM.
-
-At the time of writing the only option was `quick-xml` (v0.10), which does not support DTD and
-token positions.
-
+This library is basically a low-level XML tokenizer that preserves a position of the tokens
+and does not intend to be used directly.
 If you are looking for a more high-level solution - checkout
 [roxmltree](https://github.com/RazrFalcon/roxmltree).
 
@@ -41,6 +30,7 @@ If you are looking for a more high-level solution - checkout
 
 - Currently, only ENTITY objects are parsed from the DOCTYPE. Other ignored.
 - No tree structure validation. So an XML like `<root><child></root></child>`
+  or a string without root element
   will be parsed without errors. You should check for this manually.
   On the other hand `<a/><a/>` will lead to an error.
 - Duplicated attributes is not an error. So an XML like `<item a="v1" a="v2"/>`
@@ -56,10 +46,11 @@ If you are looking for a more high-level solution - checkout
 
 #![cfg_attr(feature = "cargo-clippy", allow(unreadable_literal))]
 
-#![doc(html_root_url = "https://docs.rs/xmlparser/0.9.0")]
+#![doc(html_root_url = "https://docs.rs/xmlparser/0.10.0")]
 
 #![forbid(unsafe_code)]
 #![warn(missing_docs)]
+#![allow(ellipsis_inclusive_range_patterns)]
 
 
 use std::fmt;
@@ -75,13 +66,11 @@ macro_rules! matches {
 }
 
 
-mod bytestream;
 mod error;
 mod stream;
 mod strspan;
 mod xmlchar;
 
-pub use bytestream::*;
 pub use error::*;
 pub use stream::*;
 pub use strspan::*;
@@ -378,12 +367,14 @@ pub struct Tokenizer<'a> {
 }
 
 impl<'a> From<&'a str> for Tokenizer<'a> {
+    #[inline]
     fn from(text: &'a str) -> Self {
         Self::from(StrSpan::from(text))
     }
 }
 
 impl<'a> From<StrSpan<'a>> for Tokenizer<'a> {
+    #[inline]
     fn from(span: StrSpan<'a>) -> Self {
         Tokenizer {
             stream: Stream::from(span),
@@ -673,7 +664,7 @@ impl<'a> Tokenizer<'a> {
                     }
                 }
             }
-            b']' if s.starts_with(b"]>") => {
+            b']' if state == State::Dtd && s.starts_with(b"]>") => {
                 s.advance(2);
                 TokenType::DoctypeEnd
             }
@@ -876,8 +867,8 @@ impl<'a> Tokenizer<'a> {
 
         let c = s.curr_byte()?;
         if c != b'[' && c !=  b'>' {
-            let chars = vec![c, b'[', b'>'];
-            return Err(StreamError::InvalidChar(chars, s.gen_text_pos()));
+            static EXPECTED: &[u8] = &[b'[', b'>'];
+            return Err(StreamError::InvalidCharMultiple(c, EXPECTED, s.gen_text_pos()));
         }
 
         s.advance(1);
@@ -985,9 +976,9 @@ impl<'a> Tokenizer<'a> {
                 }
             }
             _ => {
-                let chars = vec![c, b'"', b'\'', b'S', b'P'];
+                static EXPECTED: &[u8] = &[b'"', b'\'', b'S', b'P'];
                 let pos = s.gen_text_pos();
-                Err(StreamError::InvalidChar(chars, pos))
+                Err(StreamError::InvalidCharMultiple(c, EXPECTED, pos))
             }
         }
     }
@@ -1079,12 +1070,8 @@ impl<'a> Tokenizer<'a> {
         let (prefix, local) = s.consume_qname()?;
         s.consume_eq()?;
         let quote = s.consume_quote()?;
-        let value = s.consume_bytes(|_, c| c != quote);
-
-        if value.as_str().contains('<') {
-            return Err(StreamError::InvalidAttributeValue);
-        }
-
+        // The attribute value must not contain the < character.
+        let value = s.consume_bytes(|_, c| c != quote && c != b'<');
         s.consume_byte(quote)?;
         let span = s.slice_back(start);
 
@@ -1102,6 +1089,7 @@ impl<'a> Tokenizer<'a> {
 impl<'a> Iterator for Tokenizer<'a> {
     type Item = Result<Token<'a>>;
 
+    #[inline]
     fn next(&mut self) -> Option<Self::Item> {
         if self.stream.at_end() || self.state == State::End {
             return None;
