@@ -15,6 +15,7 @@ pub struct StrSpan<'a> {
 }
 
 impl<'a> From<&'a str> for StrSpan<'a> {
+    #[inline]
     fn from(text: &'a str) -> Self {
         StrSpan {
             text,
@@ -53,7 +54,7 @@ impl<'a> StrSpan<'a> {
     /// Returns a length of the span.
     #[inline]
     pub fn is_empty(&self) -> bool {
-        self.span.len() == 0
+        self.span.is_empty()
     }
 
     /// Returns a span slice.
